/*******************************************************************************
  FILE: RegExpValidate.js

  DESCRIPTION:
  This file contains a library of validation functions using JavaScript 
  regular expressions.  Library also contains functions that re-format 
  fields for display or for storage.

  VALIDATION FUNCTIONS:

	validateCurrency - checks for valid currency format
	validateTime - checks for valid 12 hour time
	validateState -  checks for valid state abbreviation  
	validateSSN - checks format of social security number
	validateEmail - checks format of email address
	validateUSPhone - checks format of US phone number
	validateNumeric - checks for valid numeric value
	validateInteger - checks for valid integer value
	validateNotEmpty - checks for blank form field
	validateUSZip - checks for valid US zip code
	validateUSDate - checks for valid date in US format
	validateValue - checks a string against supplied pattern
  
  FORMAT FUNCTIONS:

	rightTrim - removes trailing spaces from a string
	leftTrim - removes leading spaces from a string
	trimAll - removes leading and trailing spaces from a string
	removeCurrency - removes currency formatting characters (), $ 
	addCurrency - inserts currency formatting characters
	removeCommas - removes comma separators from a number
	addCommas - adds comma separators to a number
	removeCharacters - removes characters from a string that match passed pattern

  AUTHOR: Karen Gayda

  Karen Gayda is a Senior Software Engineer in the Internet Services Division 
  at Stellcom Technologies located in San Diego,CA. Her specialties include 
  VB COM, Active Server Pages, Dynamic HTML, SQL Server, and scripting languages.
  She can be reached at kgayda@yahoo.com

  DATE: 03/24/2000

  URL: http://www.siteexperts.com/tips/functions/ts23/page1.asp
********************************************************************************
********************************************************************************
  DESCRIPTION:
  Validates that a string contains a valid currency format. 
  
  PARAMETERS:
  strValue - String to be tested for validity
   
  RETURNS:
  True if valid, otherwise false.
*******************************************************************************/

function REV_validateCurrency( strValue)  {
	var objRegExp = /(^\$\d{1,3}(,\d{3})*\.\d{2}$)|(^\(\$\d{1,3}(,\d{3})*\.\d{2}\)$)/;
	return objRegExp.test( strValue );
}

/*******************************************************************************
  DESCRIPTION:
  Validates that a string contains a valid 12 hour time format. 
  Seconds are optional.
  
  PARAMETERS:
  strValue - String to be tested for validity
   
  RETURNS:
  True if valid, otherwise false.
*******************************************************************************/

function REV_validateTime ( strValue ) {
	var objRegExp = /^([1-9]|1[0-2]):[0-5]\d(:[0-5]\d)?$/;
	return objRegExp.test( strValue );
}

/*******************************************************************************
  DESCRIPTION:
  Validates that a string contains a valid state abbreviation. 
  
  PARAMETERS:
  strValue - String to be tested for validity
   
  RETURNS:
  True if valid, otherwise false.
*******************************************************************************/

function REV_validateState (strValue ) {
	var objRegExp = /^AK$|^AL$|^AR$|^AZ$|^CA$|^CO$|^CT$|^DC$|^DE$|^FL$|^GA$|^HI$|^IA$|^ID$|^IL$|^IN$|^KS$|^KY$|^LA$|^MA$|^MD$|^ME$|^MI$|^MN$|^MO$|^MS$|^MT$|^NB$|^NC$|^ND$|^NH$|^NJ$|^NM$|^NV$|^NY$|^OH$|^OK$|^OR$|^PA$|^RI$|^SC$|^SD$|^TN$|^TX$|^UT$|^VA$|^VT$|^WA$|^WI$|^WV$|^WY$/i;
	return objRegExp.test(strValue);
}

/*******************************************************************************
  DESCRIPTION:
  Validates that a string contains a valid social security number. 
  
  PARAMETERS:
  strValue - String to be tested for validity
   
  RETURNS:
  True if valid, otherwise false.
*******************************************************************************/

function REV_validateSSN( strValue ) {
	var objRegExp = /^\d{3}\-\d{2}\-\d{4}$/;
	return objRegExp.test(strValue);
}

/*******************************************************************************
  DESCRIPTION:
  Validates that a string contains a valid email pattern. 
  
  PARAMETERS:
  strValue - String to be tested for validity
   
  RETURNS:
  True if valid, otherwise false.
   
  REMARKS:
  Accounts for email with country appended does not validate that email 
  contains valid URL type (.com, .gov, etc.) or valid country suffix.
*******************************************************************************/

function REV_validateEmail( strValue) {
	var objRegExp = /(^[a-z]([a-z_\.]*)@([a-z_\.]*)([.][a-z]{3})$)|(^[a-z]([a-z_\.]*)@([a-z_\.]*)(\.[a-z]{3})(\.[a-z]{2})*$)/i;
	return objRegExp.test(strValue);
}

/*******************************************************************************
  DESCRIPTION:
  Validates that a string contains valid US phone pattern with or without 
  space between area code, e.g., (999) 999-9999 or (999)999-9999
  
  PARAMETERS:
  strValue - String to be tested for validity
   
  RETURNS:
  True if valid, otherwise false.
*******************************************************************************/

function REV_validateUSPhone( strValue ) {
	var objRegExp = /^\([1-9]\d{2}\)\s?\d{3}\-\d{4}$/;
	return objRegExp.test(strValue); 
}

/*******************************************************************************
  DESCRIPTION:
  Validates that a string contains only valid numbers.

  PARAMETERS:
  strValue - String to be tested for validity
   
  RETURNS:
  True if valid, otherwise false.
*******************************************************************************/

function  REV_validateNumeric( strValue ) {
	var objRegExp = /(^-?\d\d*\.\d*$)|(^-?\d\d*$)|(^-?\.\d\d*$)/; 
	return objRegExp.test(strValue);
}

/*******************************************************************************
  DESCRIPTION:
  Validates that a string contains only valid integer number.
    
  PARAMETERS:
  strValue - String to be tested for validity
   
  RETURNS:
  True if valid, otherwise false.
*******************************************************************************/

function REV_validateInteger( strValue ) {
	var objRegExp = /(^-?\d\d*$)/;
	return objRegExp.test(strValue);
}

/*******************************************************************************
  DESCRIPTION:
  Validates that a string is not all blank (whitespace) characters.
    
  PARAMETERS:
  strValue - String to be tested for validity
   
  RETURNS:
  True if valid, otherwise false.
*******************************************************************************/

function REV_validateNotEmpty( strValue ) {
	var strTemp = strValue;
	strTemp = trimAll(strTemp);
	return ( strTemp.length > 0 ? true : false );
}

/*******************************************************************************
  DESCRIPTION:
  Validates that a string a United States zip code in 5 digit format or zip+4
  format, e.g., 99999 or 99999-9999
    
  PARAMETERS:
  strValue - String to be tested for validity
   
  RETURNS:
  True if valid, otherwise false.

*******************************************************************************/

function REV_validateUSZip( strValue ) {
	var objRegExp = /(^\d{5}$)|(^\d{5}-\d{4}$)/;
	return objRegExp.test(strValue);
}

/*******************************************************************************
  DESCRIPTION:
  Validates that a string contains only valid dates with 2 digit month, 
  2 digit day, 4 digit year. Date separator can be ., -, or /. Uses 
  combination of regular expressions and string parsing to validate date, 
  e.g., mm/dd/yyyy or mm-dd-yyyy or mm.dd.yyyy
    
  PARAMETERS:
  strValue - String to be tested for validity
   
  RETURNS:
  True if valid, otherwise false.
   
  REMARKS:
  Avoids some of the limitations of the Date.parse() method such as the 
  date separator character.
*******************************************************************************/

function REV_validateUSDate( strValue ) {
	var objRegExp = /^\d{1,2}(\-|\/|\.)\d{1,2}\1\d{4}$/
	if( !objRegExp.test(strValue) )
		return false; //doesn't match pattern, bad date
	else {
		var strSeparator = strValue.substring(2,3) //find date separator
		var arrayDate = strValue.split(strSeparator); //split date into month, day, year
		//create a lookup for months not equal to Feb.
		var arrayLookup = { '01' : 31,'03' : 31, '04' : 30,'05' : 31,'06' : 30,'07' : 31,
                            '08' : 31,'09' : 30,'10' : 31,'11' : 30,'12' : 31 }
		var intDay = parseInt(arrayDate[1]); 
		//check if month value and day value agree
		if ( arrayLookup[arrayDate[0]] != null ) {
			if( intDay <= arrayLookup[arrayDate[0]] && intDay != 0 )
				return true; //found in lookup table, good date
		}
		//check for February
		var intYear = parseInt(arrayDate[2]);
		var intMonth = parseInt(arrayDate[0]);
		if( ((intYear % 4 == 0 && intDay <= 29) || (intYear % 4 != 0 && intDay <=28)) && intDay !=0)
			return true; //Feb. had valid number of days
	}
	return false; //any other values, bad date
}

/*******************************************************************************
  DESCRIPTION:
  Validates that a string a matches a valid regular expression value.
    
  PARAMETERS:
  strValue - String to be tested for validity
  strMatchPattern - String containing a valid regular expression match pattern.
      
  RETURNS:
  True if valid, otherwise false.
*******************************************************************************/

function REV_validateValue( strValue, strMatchPattern ) {
	var objRegExp = new RegExp( strMatchPattern );
	return objRegExp.test(strValue);
}

/*******************************************************************************
  DESCRIPTION:
  Trims trailing whitespace characters.
    
  PARAMETERS:
  strValue - String to be trimmed.  
      
  RETURNS:
  Source string with right whitespaces removed.
*******************************************************************************/

function REV_rightTrim( strValue ) {
	var objRegExp = /^([\w\W]*)(\b\s*)$/;
 	if ( objRegExp.test(strValue) ) {
		//remove trailing a whitespace characters
		strValue = strValue.replace(objRegExp, '$1');
	}
	return strValue;
}

/*******************************************************************************
  DESCRIPTION:
  Trims leading whitespace characters.
    
  PARAMETERS:
  strValue - String to be trimmed
   
  RETURNS:
  Source string with left whitespaces removed.
*******************************************************************************/

function REV_leftTrim( strValue ) {
	var objRegExp = /^(\s*)(\b[\w\W]*)$/;
	if ( objRegExp.test(strValue) ) {
		//remove leading a whitespace characters
		strValue = strValue.replace(objRegExp, '$2');
	}
	return strValue;
}

/*******************************************************************************
  DESCRIPTION:
  Removes leading and trailing spaces.

  PARAMETERS:
  Source string from which spaces will be removed;

  RETURNS:
  Source string with whitespaces removed.
*******************************************************************************/ 

function REV_trimAll( strValue ) {
	var objRegExp = /^(\s*)$/;
	//check for all spaces
    if ( objRegExp.test(strValue) ) {
		strValue = strValue.replace(objRegExp, '');
		if( strValue.length == 0) return strValue;
	}

	//check for leading & trailing spaces
	objRegExp = /^(\s*)([\W\w]*)(\b\s*$)/;
	if ( objRegExp.test(strValue) ) {
		//remove leading and trailing whitespace characters
		strValue = strValue.replace(objRegExp, '$2');
	}
	return strValue;
}

/*******************************************************************************
  DESCRIPTION:
  Removes currency formatting from source string.
  
  PARAMETERS:
  strValue - Source string from which currency formatting will be removed.

  RETURNS:
  Source string with commas removed.
*******************************************************************************/

function REV_removeCurrency( strValue ) {
	var objRegExp = /\(/;
	var strMinus = '';
 
	//check if negative
	if ( objRegExp.test(strValue) ){
		strMinus = '-';
	}

	objRegExp = /\)|\(|[,]/g;
	strValue = strValue.replace(objRegExp,'');
	if ( strValue.indexOf('$') >= 0 ) {
		strValue = strValue.substring(1, strValue.length);
	}
	return strMinus + strValue;
}

/*******************************************************************************
  DESCRIPTION:
  Formats a number as currency.

  PARAMETERS: 
  strValue - Source string to be formatted.

  REMARKS:
  Assumes number passed is a valid numeric value in the rounded to 2 decimal 
  places. If not, returns original value.
*******************************************************************************/

function REV_addCurrency( strValue ) {
	var objRegExp = /-?[0-9]+\.[0-9]{2}$/;
	if ( objRegExp.test(strValue) ) {
		objRegExp.compile('^-');
		strValue = addCommas(strValue);
		if ( objRegExp.test(strValue) ) {
			strValue = '($' + strValue.replace(objRegExp,'') + ')';
		}
		else {
        	strValue = '$' + strValue;
		}
	}
	return strValue;
}

/*******************************************************************************
  DESCRIPTION:
  Removes commas from source string.

  PARAMETERS: 
  strValue - Source string from which commas will be removed.

  RETURNS:
  Source string with commas removed.
*******************************************************************************/

function REV_removeCommas( strValue ) {
	var objRegExp = /,/g;					//search for commas globally
	return strValue.replace(objRegExp,'');
}

/*******************************************************************************
  DESCRIPTION:
  Inserts commas into numeric string.

  PARAMETERS: 
  strValue - Source string containing no commas.
  
  RETURNS:
  String modified with comma grouping if source was all numeric, otherwise 
  source is returned.
  
  REMARKS:
  Used with integers or numbers with 2 or less decimal places.
*******************************************************************************/

function REV_addCommas( strValue ) {
	var objRegExp = new RegExp('(-?[0-9]+)([0-9]{3})'); 

	//check for match to search criteria
	while( objRegExp.test(strValue) ) {
		// replace original string with first group match, a comma, 
		// then second group match
		strValue = strValue.replace(objRegExp, '$1,$2');
	}
	return strValue;
}

/*******************************************************************************
  DESCRIPTION:
  Removes characters from a source string based upon matches of the 
  supplied pattern.

  PARAMETERS: 
  strValue - Source string containing number.
  
  RETURNS:
  String modified with characters matching search pattern removed.
  
  USAGE:
  strNoSpaces = removeCharacters( ' sfdf  dfd', '\s*' )
*******************************************************************************/

function REV_removeCharacters( strValue, strMatchPattern ) {
	var objRegExp = new RegExp( strMatchPattern, 'gi' );
	return strValue.replace(objRegExp,'');
}
